from functools import partial


def filter_since(since_date, stat):
    return since_date <= stat['date']


def filter_until(until_date, stat):
    return stat['date'] <= until_date


def filter_disks(disk_names, stat):
    filtered = []
    for disk in stat['device']['stats']:
        for name in disk:
            if name in disk_names:
                filtered.append(disk)
    stat['device']['stats'] = filtered
    return True


def get_filters(args):
    filters = []
    if args.since is not None:
        filters.append(partial(filter_since, args.since))
    if args.until is not None:
        filters.append(partial(filter_until, args.until))
    if args.disks:
        filters.append(partial(filter_disks, args.disks))
    return filters



import math
from collections import defaultdict

from matplotlib import dates as mdates
from matplotlib import gridspec
from matplotlib import pyplot as plt

from .consts import AVGRQ_SZ, AVGQU_SZ, AWAIT, SVCTM
from .consts import IO_RQM, IOPS, IO_TRANSFER, PERCENT_UTIL
from .consts import PERCENT_IO_RQM
from .renderer import Renderer
from .utils import get_logger

log = get_logger()
default_figsize = plt.rcParams.get('figure.figsize')


class Plotter(Renderer):

    def __init__(self, args, stats):
        self.args = args
        self.stats = stats
        self.subplot_borderaxespad = -1

        figsize = args.figsize
        if figsize is None:
            figsize = (18, 14)

        self.fig = plt.figure(figsize=figsize)
        self.fig.suptitle(self.args.title)

        if self.args.cpu_only:
            self.args.subplots = []

        if self.args.with_cpu:
            add_rows = 1
        else:
            add_rows = 0

        self._update_args_subplots()
        row_length = math.ceil(len(self.args.subplots) / 2.0)
        gs = gridspec.GridSpec(row_length + add_rows, 2, wspace=0.4)

        if args.with_cpu:
            self.cpu = self.fig.add_subplot(gs[0, :])
            self.cpu.set_title('cpu average')
            self.cpu.set_ylim(0, 100)
            self.cpu.set_ylabel('percent')

        self.subplots = {}
        gs_range = [(i, j) for j in (0, 1)
                    for i in range(add_rows, row_length + add_rows)]
        for i, (row, column) in enumerate(gs_range):
            try:
                name = self.args.subplots[i]
            except IndexError:
                break
            if len(self.args.subplots) == 1:
                subplot = self.fig.add_subplot(gs[row, :])
                self.subplot_borderaxespad = -3
            else:
                subplot = self.fig.add_subplot(gs[row, column])
            self.set_device_subplot_params(name, subplot)
            if self.args.x_datetime_format is not None:
                x_format = mdates.DateFormatter(self.args.x_datetime_format)
                subplot.xaxis.set_major_formatter(x_format)
            self.subplots[name] = subplot

    def _update_args_subplots(self):
        if self.args.cpu_only:
            return
        if not self.has_stat_data('device'):
            return
        columns = self.stats[0]['device']['columns']
        # TODO: check stats columns for all cases
        #       only PERCENT_IO_RQM in new iostat outputs at this time
        for col in ['%rrqm', '%wrqm']:
            if col in columns:
                return
        self.args.subplots.remove(PERCENT_IO_RQM)

    def set_device_subplot_params(self, name, subplot):
        if name == IO_RQM:
            subplot.set_title('io merged request counts per second')
            subplot.set_ylabel('counts')
        elif name == PERCENT_IO_RQM:
            subplot.set_title('percentage of requests merged together')
            subplot.set_ylabel('percent')
        elif name == IOPS:
            subplot.set_title('iops')
            subplot.set_ylabel('counts')
        elif name == IO_TRANSFER:
            subplot.set_title('io transer per second')
            subplot.set_ylabel('[M|K]Byte or Sector')
        elif name == PERCENT_UTIL:
            subplot.set_title('%util')
            subplot.set_ylabel('percent')
        elif name == AVGRQ_SZ:
            subplot.set_title('average size (in sectors) of the requests')
            subplot.set_ylabel('sectors')
        elif name == AVGQU_SZ:
            subplot.set_title('average queue length of the requests')
            subplot.set_ylabel('length')
        elif name == AWAIT:
            subplot.set_title('average time for i/o requests')
            subplot.set_ylabel('milliseconds')
        elif name == SVCTM:
            subplot.set_title('average service time')
            subplot.set_ylabel('milliseconds')
        else:
            raise NotImplementedError('unsupported subplot: %s' % name)

    def has_stat_data(self, target):
        return self.stats[0][target]['columns'] is not None

    def plot_cpu(self, x):
        if not self.has_stat_data('cpu'):
            return

        data = defaultdict(list)
        for stat in self.stats:
            cpu = stat['cpu']
            for i, column in enumerate(cpu['columns']):
                data[column].append(cpu['stat'][i])

        for column, values in data.items():
            self.cpu.plot(x, values, label=column)

        for vline in self.args.vlines:
            self.cpu.axvline(vline, linestyle=':', linewidth=3, color='purple')
        self.cpu.legend(
            bbox_to_anchor=(1.04, 0.5), loc='center left', borderaxespad=-3)

    def set_device_data(self, data, device):
        def set_data_value(data, columns, disk_stat_data):
            for col in columns:
                value = disk_stat_data.get(col)
                if value is not None:
                    data[name][disk_name + '_' + col].append(value)

        _disk_stat_data = {}
        for disk in device['stats']:
            for disk_name, disk_stat in disk.items():
                for i, column in enumerate(device['columns']):
                    _disk_stat_data[column] = disk_stat[i]
                for name in data:
                    if name == IO_RQM:
                        columns = ['rrqm/s', 'wrqm/s']
                    elif name == PERCENT_IO_RQM:
                        columns = ['%rrqm', '%wrqm']
                    elif name == IOPS:
                        columns = ['r/s', 'w/s']
                    elif name == IO_TRANSFER:
                        columns = [
                            'rMB/s', 'wMB/s',    # iostat -m
                            'rkB/s', 'wkB/s',    # iostat -k
                            'rsec/s', 'wsec/s',  # by default
                        ]
                    elif name == PERCENT_UTIL:
                        columns = ['%util']
                    elif name == AVGRQ_SZ:
                        columns = [
                            'avgrq-sz', 'areq-sz', 'rareq-sz', 'wareq-sz'
                        ]
                    elif name == AVGQU_SZ:
                        columns = ['avgqu-sz', 'aqu-sz']
                    elif name == AWAIT:
                        columns = ['await', 'r_await', 'w_await']
                    elif name == SVCTM:
                        columns = ['await', 'svctm']
                    else:
                        assert False
                    set_data_value(data, columns, _disk_stat_data)

    def plot_device(self, x):
        if not self.has_stat_data('device'):
            return

        data = {}
        for name in self.subplots:
            data[name] = defaultdict(list)

        for stat in self.stats:
            self.set_device_data(data, stat['device'])

        for name, device_data in data.items():
            for column, values in device_data.items():
                self.subplots[name].plot(x, values, label=column)

            for vline in self.args.vlines:
                self.subplots[name].axvline(
                    vline, linestyle=':', linewidth=3, color='purple',
                )
            self.subplots[name].legend(
                bbox_to_anchor=(1.04, 0.5), loc='center left',
                borderaxespad=self.subplot_borderaxespad
            )

    def plot(self):
        datetime_data = [i['date'] for i in self.stats]
        if not self.args.cpu_only:
            self.plot_device(datetime_data)
        if self.args.with_cpu:
            self.plot_cpu(datetime_data)
        if not self.args.cpu_only and self.args.with_cpu:
            plt.subplots_adjust(hspace=0.4)

    def show(self):
        plt.show(block=True)

    def save(self):
        plt.savefig(self.output)



from datetime import datetime, timedelta

import matplotlib.pyplot as plt
from matplotlib import gridspec

from .renderer import Renderer


class Scatter(Renderer):

    def __init__(self, args):
        self.args = args
        self.closed = False
        self.start_date = args.since
        if self.start_date is None:
            self.start_date = datetime.now()

        figsize = args.figsize
        if figsize is None:
            figsize = (18, 13)

        self.fig = plt.figure(figsize=figsize)
        self.fig.suptitle('iostat scatter')
        self.fig.canvas.mpl_connect('close_event', self.close_handler)

        self.row_length = 5
        self.init_cpu()
        self.init_device()

    def close_handler(self, event):
        # FIXME: how to exit forcely
        self.closed = True
        plt.close(self.fig)
        raise SystemExit

    def init_cpu(self):
        self.cpus = []
        items = ['%user', '%nice', '%system', '%iowait', '%steal', '%idle']
        for i, item in enumerate(items, 1):
            subplot = self.fig.add_subplot(self.row_length, 6, i)
            subplot.set_title(item)
            subplot.set_ylabel('percent')
            self.cpus.append(subplot)

    def init_device(self):
        items = [
            'rrqm/s', 'wrqm/s', 'r/s', 'w/s',
            'r[MB|kB|sec]/s', 'w[MB|kB|sec]/s', 'avgrq-sz', 'avgqu-sz',
            'await', 'r_await', 'w_await', 'svctm',
        ]

        step = 4
        self.devices = []
        for num in range(0, len(items), step):
            for i, item in enumerate(items[num:num + step], 1):
                loc = i + num + step
                subplot = self.fig.add_subplot(self.row_length, step, loc)
                subplot.set_title(item)
                self.devices.append(subplot)

        gs = gridspec.GridSpec(self.row_length, 6)
        subplot = self.fig.add_subplot(gs[self.row_length - 1, :])
        subplot.set_title('%util')
        self.devices.append(subplot)

    def scatter_cpu(self, date, stat):
        if stat['columns'] is None:
            return

        end_date = date + timedelta(seconds=2)
        for i, column in enumerate(stat['columns']):
            self.cpus[i].scatter(date, stat['stat'][i])
            self.cpus[i].axis([self.start_date, end_date, 0, 100])

    def scatter_device(self, date, device):
        if device['columns'] is None:
            return

        end_date = date + timedelta(seconds=2)
        for i, column in enumerate(device['columns']):
            for disk in device['stats']:
                for disk_name, disk_stat in disk.items():
                    self.devices[i].set_xlim(self.start_date, end_date)
                    self.devices[i].scatter(date, disk_stat[i])

    def scatter(self, stat):
        if self.closed:
            return

        date = stat['date']
        self.scatter_cpu(date, stat['cpu'])
        self.scatter_device(date, stat['device'])
        plt.pause(0.001)
        plt.subplots_adjust(wspace=0.4, hspace=0.4)

    def set_vlines(self):
        if not hasattr(self.args, 'vlines'):
            return

        for vline in self.args.vlines:
            for cpu in self.cpus:
                cpu.axvline(vline, linestyle=':', linewidth=3, color='purple')
            for dev in self.devices:
                dev.axvline(vline, linestyle=':', linewidth=3, color='purple')

    def show(self):
        self.set_vlines()
        plt.show(block=True)

    def save(self):
        self.set_vlines()
        plt.savefig(self.output)



import matplotlib.pyplot as plt

from .utils import make_output_file


class Renderer:
    """
    Base class
    """
    def __init__(self, args):
        self.args = args

    @property
    def output(self):
        output = self.args.figoutput
        if output is None:
            base_path = self.args.data
            if base_path is None:
                base_path = 'iostat'
            output = make_output_file(base_path, 'png')
        return output

    def close_handler(self, event):
        plt.close(self.fig)
        raise SystemExit

    def render(self):
        if self.args.backend == 'Agg':
            self.save()
        else:
            self.show()
        plt.close(self.fig)



"""
Running process asynchronously supports over Python 3.4,
so use generator based coroutine instead of native coroutine (async/await)
"""
import asyncio
import asyncio.subprocess

from .parser import Parser
from .scatter import Scatter
from .utils import get_logger

log = get_logger()


async def run_process(future, command_and_args, queue):
    create = asyncio.create_subprocess_exec(
        *command_and_args,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.STDOUT
    )
    proc = await create
    while True:
        line = await proc.stdout.readline()
        if queue.full():
            await asyncio.sleep(1)
        else:
            queue.put_nowait(line)

        # check process is finished or not,
        # but it might not occurred immediately
        # in that case, proc.stdout.readline() might return empty line
        # during several seconds
        if proc.returncode is not None:
            break

    future.set_result(proc.returncode)


async def read_stream(queue, args):
    parser = Parser(args)
    scatter = Scatter(args)
    with open(args.output, 'wb') as f:
        try:
            while True:
                if queue.empty():
                    await asyncio.sleep(1)
                else:
                    # TODO: it might miss rest of last queue entries
                    #       when the process finished
                    output = queue.get_nowait()
                    f.write(output)
                    line = output.decode()
                    print(line.rstrip())

                    for stat in parser.parse_line(line):
                        # note: get stat for previous date entry
                        if parser.filter(stat):
                            scatter.scatter(stat)
        except Exception:
            if args.backend == 'Agg':
                scatter.save()


def cancel_tasks():
    # TODO: confirm detailed behavior
    for task in asyncio.Task.all_tasks():
        task.cancel()


def finish_tasks(future):
    status_code = future.result()
    if status_code != 0:
        log.error('process was not finished normally: %d', status_code)
    cancel_tasks()


def run_iostat(args):
    command_and_args = ['iostat'] + args.iostat_args.split()
    log.debug(command_and_args)
    event_loop = asyncio.get_event_loop()
    future = asyncio.Future()
    queue = asyncio.Queue(maxsize=args.max_queue_size, loop=event_loop)
    asyncio.ensure_future(run_process(future, command_and_args, queue))
    future.add_done_callback(finish_tasks)
    try:
        event_loop.run_until_complete(read_stream(queue, args))
        event_loop.run_forever()
    except KeyboardInterrupt:
        cancel_tasks()
    except asyncio.CancelledError:
        pass
    finally:
        event_loop.close()



import csv
from contextlib import ContextDecorator

from .utils import add_suffix_to_name


class BaseWriter(ContextDecorator):

    def __init__(self, args, suffix):
        self.args = args
        output = add_suffix_to_name(args.output, suffix)
        self.f = open(output, 'w')
        self.writer = csv.writer(
            self.f, dialect=args.dialect, delimiter=args.separator,
            quoting=csv.QUOTE_MINIMAL
        )

    def __enter__(self):
        return self

    def __exit__(self, *exc):
        self.f.close()

    def write(self, row):
        self.writer.writerow(row)


class CPUWriter(BaseWriter):

    def __init__(self, args):
        suffix = 'cpu'
        super().__init__(args, suffix)


class DeviceWriter(BaseWriter):

    def __init__(self, args):
        suffix = 'devices'
        super().__init__(args, suffix)

    def write_rows(self, date, stats):
        for disk_stat in stats:
            for name, stat in disk_stat.items():
                row = [date, name]
                row.extend(stat)
                self.writer.writerow(row)


def write_csv(args, parser):
    def make_row(date, stat):
        row = [date]
        row.extend(stat)
        return row

    g = parser.parse()
    first_row = next(g)
    with CPUWriter(args) as cpu, DeviceWriter(args) as device:
        # write first row after csv header
        cpu.write(make_row('datetime', first_row['cpu']['columns']))
        cpu.write(make_row(first_row['date'], first_row['cpu']['stat']))
        device.write(['datetime', 'device'] + first_row['device']['columns'])
        device.write_rows(first_row['date'], first_row['device']['stats'])
        # sencod rows
        for stat in g:
            cpu.write(make_row(stat['date'], stat['cpu']['stat']))
            device.write_rows(stat['date'], stat['device']['stats'])



import os

from .filters import get_filters
from .utils import get_iostat_date_format
from .utils import get_logger
from .utils import parse_datetime

log = get_logger()


class Parser:

    CPU = 'CPU'
    DATE = 'DATE'
    DEVICE = 'Device'

    def __init__(self, args):
        self.args = args
        self.extra_lines = []
        self.state = None
        self.filters = get_filters(args)
        self.init_stat()

    def init_stat(self):
        self.date = None
        self.cpu_stat = {'columns': None, 'stat': None}
        self.device_stat = {'columns': None, 'stats': []}

    def make_stat(self):
        stat = {
            'date': self.date,
            'cpu': self.cpu_stat,
            'device': self.device_stat,
        }
        self.init_stat()
        return stat

    def parse_cpu_stat(self, line):
        self.cpu_stat['stat'] = [float(i) for i in line.split()]

    def parse_device_stat(self, line):
        s = line.split()
        stat = {s[0]: [float(i) for i in s[1:]]}
        self.device_stat['stats'].append(stat)

    def parse_columns(self, d, line):
        d['columns'] = line.strip().split()[1:]

    def _parse(self, line):
        if line == '\n':
            self.state = None
            return

        line = line.strip()
        if self.state == self.CPU:
            self.parse_cpu_stat(line)
        elif self.state == self.DEVICE:
            self.parse_device_stat(line)
        else:
            date_format = get_iostat_date_format(line)
            if date_format is not None:
                if self.date is not None:
                    yield self.make_stat()

                self.init_stat()
                self.date = parse_datetime(line, fmt=date_format)
                self.state = self.DATE
            else:
                if line.startswith('avg-cpu:'):
                    self.parse_columns(self.cpu_stat, line)
                    self.state = self.CPU
                elif line.startswith('Device'):
                    self.parse_columns(self.device_stat, line)
                    self.state = self.DEVICE
                else:
                    log.debug('not handled line: %s', line)
                    self.extra_lines.append(line)

    def parse_line(self, line):
        yield from self._parse(line)

    def parse_all(self):
        if not os.path.isfile(self.args.data):
            log.error('target file is not found: %s', self.args.data)
            return

        with open(self.args.data) as f:
            for line in f:
                yield from self._parse(line)
            yield self.make_stat()  # last stat data

    def filter(self, stat):
        for filter_func in self.filters:
            if not filter_func(stat):
                return False
        return True

    def parse(self):
        for stat in self.parse_all():
            if self.filter(stat):
                yield stat



PACKAGE_NAME = 'iostat-tool'

# parser options
SUB_COMMAND_CSV = 'csv'
SUB_COMMAND_MONITOR = 'monitor'
SUB_COMMAND_PLOT = 'plot'

# plot options
PLOT_TYPE_PLOTTER = 'plotter'
PLOT_TYPE_SCATTER = 'scatter'
PLOT_TYPES = [
    PLOT_TYPE_PLOTTER,
    PLOT_TYPE_SCATTER,
]

# device subplots parameters
IO_RQM = 'io_rqm'
PERCENT_IO_RQM = '%io_rqm'
IOPS = 'iops'
IO_TRANSFER = 'io_transfer'
PERCENT_UTIL = '%util'
AVGRQ_SZ = 'avgrq-sz'
AVGQU_SZ = 'avgqu-sz'
AWAIT = 'await'
SVCTM = 'svctm'

DEVICE_SUBPLOTS = [
    IO_RQM, PERCENT_IO_RQM,
    IOPS, IO_TRANSFER, PERCENT_UTIL,
    AVGRQ_SZ, AVGQU_SZ, AWAIT, SVCTM,
]



import os
import re

from setuptools import setup

this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

main_py = open('iostat/main.py').read()
metadata = dict(re.findall("__([a-z]+)__ = '([^']+)'", main_py))

setup(
    name='iostat-tool',
    version=metadata['version'],
    description='parse and visualize iostat output',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries',
    ],
    keywords=['iostat'],
    author='Tetsuya Morimoto',
    author_email='tetsuya.morimoto@gmail.com',
    url='https://github.com/t2y/iostat-tool',
    license='Apache License 2.0',
    platforms=['unix', 'linux', 'osx', 'windows'],
    packages=['iostat'],
    include_package_data=True,
    install_requires=['matplotlib'],
    tests_require=['tox', 'pytest', 'pytest-pep8', 'pytest-flakes'],
    entry_points={
        'console_scripts': [
            'iostat-cli=iostat.main:main',
        ],
    },
)



import argparse
import csv
import logging

from .consts import DEVICE_SUBPLOTS
from .consts import PLOT_TYPES, PLOT_TYPE_PLOTTER, PLOT_TYPE_SCATTER
from .consts import SUB_COMMAND_CSV, SUB_COMMAND_MONITOR, SUB_COMMAND_PLOT
from .csv import write_csv
from .parser import Parser
from .utils import get_logger
from .utils import parse_datetime

__version__ = '0.3.1'
_DATETIME_FORMAT_HELP = 'yyyymmddHHMISS'

_COMMA = 'comma'
_TAB = 'tab'

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s %(levelname)s: %(message)s',
)
log = get_logger()


def figsize_type(s):
    values = s.split(',')
    if len(values) != 2:
        msg = 'set width and height in inches, e.g.) "6.4, 4.8"'
        raise argparse.ArgumentTypeError(msg)
    return tuple(float(i.strip()) for i in values)


def dt_type(s):
    return parse_datetime(s)


def sep_type(s):
    if s == _COMMA:
        return ','
    elif s == _TAB:
        return '\t'
    else:
        raise argparse.ArgumentTypeError('separator is wrong')


def parse_csv_argument(subparsers):
    csv_parser = subparsers.add_parser(SUB_COMMAND_CSV)
    csv_parser.set_defaults(
        dialect='excel',
        separator=_COMMA,
    )
    csv_parser.add_argument(
        '--dialect', action='store', choices=csv.list_dialects(),
        help='set dialect for csv writer, default is excel'
    )
    csv_parser.add_argument(
        '--separator', action='store', type=sep_type,
        help='set separator (choose from comma, tab), default is comma'
    )


def parse_monitor_argument(subparsers):
    monitor_parser = subparsers.add_parser(SUB_COMMAND_MONITOR)
    monitor_parser.set_defaults(
        iostat_args='',
        max_queue_size=256,
    )
    monitor_parser.add_argument(
        '--iostat-args', action='store', dest='iostat_args',
        help='set arguments for iostat'
    )
    monitor_parser.add_argument(
        '--max-queue-size', action='store', dest='max_queue_size', type=int,
        help='set queue size to read iostat output'
    )


def parse_plot_argument(subparsers):
    plot_parser = subparsers.add_parser(SUB_COMMAND_PLOT)
    plot_parser.set_defaults(
        plot_type=PLOT_TYPE_PLOTTER,
        subplots=DEVICE_SUBPLOTS,
        vlines=[],
        x_datetime_format=None,
    )

    plot_parser.add_argument(
        '--plot-type', action='store', dest='plot_type', choices=PLOT_TYPES,
        help='set plot type ("%s" by default) ' % PLOT_TYPE_PLOTTER
    )
    plot_parser.add_argument(
        '--subplots', action='store', nargs='+', choices=DEVICE_SUBPLOTS,
        help='set subplots to show'
    )
    plot_parser.add_argument(
        '--vlines', action='store', nargs='+', type=dt_type,
        help='set vertical line, format: %s' % _DATETIME_FORMAT_HELP
    )
    plot_parser.add_argument(
        '--x-datetime-format', action='store', dest='x_datetime_format',
        help='set datetime format for devices x-axis'
    )
    plot_parser.add_argument(
        '--title', action='store', dest='title', default='iostat output',
        help='set title for graph'
    )
    group = plot_parser.add_mutually_exclusive_group()
    group.add_argument(
        '--without-cpu', dest='with_cpu', action='store_false',
        help='don\'t plot CPU data'
    )
    group.add_argument(
        '--cpu-only', dest='cpu_only', action='store_true',
        help='plot only CPU data'
    )


def parse_argument():
    parser = argparse.ArgumentParser()
    parser.set_defaults(
        backend='Agg',
        data=None,
        figoutput=None,
        figsize=None,
        output='iostat.log',
        # filter options
        disks=[],
        with_cpu=True,
        cpu_only=False,
        since=None,
        until=None,
        subcommand=SUB_COMMAND_MONITOR,
    )

    parser.add_argument(
        '--backend', action='store',
        help='set backend for matplotlib, '
             'use TkAgg to monitor in the foreground',
    )
    parser.add_argument(
        '--data', action='store',
        help='set path to iostat output file',
    )
    parser.add_argument(
        '--fig-output', action='store', dest='figoutput',
        help='set path to save graph'
    )
    parser.add_argument(
        '--fig-size', action='store', dest='figsize', type=figsize_type,
        help='set figure size'
    )
    parser.add_argument(
        '--output', action='store',
        help='set path to save output of iostat',
    )

    # filter options
    parser.add_argument(
        '--disks', action='store', nargs='+',
        help='set disk name in iostat'
    )
    parser.add_argument(
        '--since', action='store', type=dt_type,
        help='set since datetime, format: %s' % _DATETIME_FORMAT_HELP
    )
    parser.add_argument(
        '--until', action='store', type=dt_type,
        help='set until datetime, format: %s' % _DATETIME_FORMAT_HELP
    )

    subparsers = parser.add_subparsers(dest='subcommand')
    subparsers.required = True
    parse_csv_argument(subparsers)
    parse_monitor_argument(subparsers)
    parse_plot_argument(subparsers)

    # for debug
    parser.add_argument(
        '-v', '--verbose', action='store_true',
        help='set verbose mode')
    parser.add_argument(
        '--version', action='version', version='%%(prog)s %s' % __version__,
        help='show program version',
    )

    args = parser.parse_args()
    if args.verbose:
        log.setLevel(logging.DEBUG)

    return args


def main():
    args = parse_argument()
    log.debug(args)

    import matplotlib
    matplotlib.use(args.backend)

    if args.subcommand == SUB_COMMAND_MONITOR:
        from .process import run_iostat
        run_iostat(args)
    else:
        if args.data is None:
            log.error('set target file with "--data path/to/file"')
            return

        parser = Parser(args)
        if args.subcommand == SUB_COMMAND_CSV:
            write_csv(args, parser)
        elif args.subcommand == SUB_COMMAND_PLOT:
            stats = [stat for stat in parser.parse()]
            if args.plot_type == PLOT_TYPE_PLOTTER:
                from .plotter import Plotter
                plotter = Plotter(args, stats)
                plotter.plot()
                plotter.render()
            elif args.plot_type == PLOT_TYPE_SCATTER:
                from .scatter import Scatter
                scatter = Scatter(args)
                for stat in stats:
                    scatter.scatter(stat)
                scatter.render()


if __name__ == '__main__':
    main()



import logging
import os
import re
from datetime import datetime

from .consts import PACKAGE_NAME


def get_logger():
    return logging.getLogger(PACKAGE_NAME)


def parse_datetime(s, fmt='%Y%m%d%H%M%S'):
    """
    >>> parse_datetime('20170403153428')
    datetime.datetime(2017, 4, 3, 15, 34, 28)

    >>> parse_datetime('2017-04-03T15:34:28', fmt='%Y-%m-%dT%H:%M:%S')
    datetime.datetime(2017, 4, 3, 15, 34, 28)

    >>> parse_datetime('06/13/2018 02:10:50 PM', fmt='%m/%d/%Y %I:%M:%S %p')
    datetime.datetime(2018, 6, 13, 14, 10, 50)

    >>> parse_datetime('09/26/21 13:35:19', fmt='%m/%d/%y %H:%M:%S')
    datetime.datetime(2021, 9, 26, 13, 35, 19)
    """
    return datetime.strptime(s, fmt)


IOSTAT_DATE_FORMAT_EN = '%m/%d/%Y %I:%M:%S %p'
IOSTAT_DATE_EN = re.compile(r"""
(?P<date>^\d{2}/\d{2}/\d{4}\s*\d{2}:\d{2}:\d{2}\s*(AM|PM))
""", re.VERBOSE)

IOSTAT_DATE_FORMAT_JA = '%m/%d/%y %H:%M:%S'
IOSTAT_DATE_JA = re.compile(r"""
(?P<date>^\d{2}/\d{2}/\d{2}\s*\d{2}:\d{2}:\d{2})
""", re.VERBOSE)


def get_iostat_date_format(s):
    """
    >>> get_iostat_date_format('06/13/2018 02:10:50 PM')
    '%m/%d/%Y %I:%M:%S %p'
    >>> get_iostat_date_format('09/26/21 13:35:19')
    '%m/%d/%y %H:%M:%S'
    >>> get_iostat_date_format('09/26/2021 03:35:19') is None
    True
    """
    en = re.search(IOSTAT_DATE_EN, s)
    if en is not None:
        return IOSTAT_DATE_FORMAT_EN
    ja = re.search(IOSTAT_DATE_JA, s)
    if ja is not None:
        return IOSTAT_DATE_FORMAT_JA
    return None


def add_suffix_to_name(path, suffix):
    """
    >>> add_suffix_to_name('sample.log', 'test')
    'sample_test.log'
    >>> add_suffix_to_name('/path/to/name', 'test')
    '/path/to/name_test'
    >>> add_suffix_to_name('/path/to/sample.log', 'test')
    '/path/to/sample_test.log'
    """
    basename = os.path.basename(path)
    names = basename.split('.')
    names[0] = '%s_%s' % (names[0], suffix)
    name = '.'.join(names)
    return os.path.join(os.path.dirname(path), name)


def make_output_file(path, ext):
    """
    >>> make_output_file('path/to/sample.data', 'csv')
    'sample.csv'
    >>> make_output_file('path/to/sample', 'png')
    'sample.png'
    >>> make_output_file('sample.data', 'png')
    'sample.png'
    """
    filename = os.path.basename(path)
    names = filename.split('.')
    if len(names) > 0:
        name = names[0]
    return '%s.%s' % (name, ext)
